include "Vram.inc"
include "VVram.inc"

ext Chars_, Ascii_
ext VVram_

VramWidth equ 24
VramHeight equ 23
CharHeight equ 8
VramRowSize equ VramWidth*CharHeight
VramTop equ Vram


ReadPlain equ 0f1h
WritePlain equ 0f2h


; void InitColors();
cseg
InitColors_: public InitColors_
    push af
        ld a,44h
        out (0f5h),a
        out (0f8h),a
        ld a,11h
        out (0f6h),a
        out (0f9h),a
        ld a,22h
        out (0f7h),a
        out (0fah),a
    pop af
ret


; void ClearScreen();
cseg
ClearScreen_: public ClearScreen_
    push af | push hl | push de | push bc
        ld a,0ffh
        out (WritePlain),a

        ld hl,Vram
        ld de,Vram+1
        ld bc,VramRowSize*VramHeight-1
        ld (hl),0
        ldir

        ld hl,Backup
        ld de,Backup+1
        ld bc,VVramWidth*VVramHeight-1
        ld (hl),0
        ldir
    pop bc | pop de | pop hl | pop af
ret


PatternAddress:
    push de
        ld l,a
        ld h,0
        ld e,l
        ld d,h
        add hl,hl ;*2
        add hl,de ;*3
        add hl,hl ;*6
        add hl,hl ;*12
        add hl,hl ;*24
        ld de,Chars_
        add hl,de
    pop de
ret


; word Put(word vram, byte c);
Put_: public Put_
    push af | push hl | push de | push bc
        ld a,e
        ex de,hl
            call PatternAddress
        ex de,hl
        ld bc,0301h
        do
            ld a,c | out (WritePlain),a
            push bc
                ld b,CharHeight
                do
                    ld a,(de) | inc de
                    ld (hl),a
                    push bc
                        ld bc,VramWidth
                        add hl,bc
                    pop bc
                dwnz
                ld bc, -VramWidth*CharHeight
                add hl,bc
            pop bc
            sla c
        dwnz
    pop bc | pop de | pop hl | pop af
    inc hl
ret


dseg
PlainBit:
    defb 0
PlainIndex:
    defb 0

cseg
PutShift:
    ex de,hl
        call PatternAddress
    ex de,hl
    ld a,(PlainBit)
    ld c,a                       
    ld b,0
    do
        ld a,c
        out (WritePlain),a
        ld a,(PlainIndex)
        add a,b
        out (ReadPlain),a
        push bc
            ld b,CharHeight
            do
                push bc
                    ld a,(de) | inc de
                    push de
                        ld e,a | ld d,0
                        ld a,i
                        ld bc,0ffc0h
                        or a
                        do
                        while nz
                            sla e | rl d
                            sla c | rl b
                            set 0,c
                            dec a
                        wend
                        ld a,(hl)
                        and c
                        or e
                        ld (hl),a
                        inc hl
                        ld a,(hl)
                        and b
                        or d
                        ld (hl),a

                        push bc
                            ld bc,VramWidth-1
                            add hl,bc
                        pop bc
                    pop de
                pop bc
            dwnz
            ld bc, -VramWidth*CharHeight
            add hl,bc
        pop bc
        sla c
        inc b
        ld a,b
        cp 3
    while nz | wend
ret


; void VVramToVram();
VVramToVram_: public VVramToVram_
    push af | push hl | push de | push bc
        ld de,VVram_
        ld hl,Backup
        exx
            ld hl,VramTop
        exx
        ld a,1 | ld (PlainBit),a
        ld (PlainIndex),a
        
        ld c,VVramHeight
        do
            push bc
                ld bc,VVramWidth shl 8
                do 
                    push bc 
                        ld a,(de)
                        inc de
                        cp (hl)
                        if nz
                            ld (hl),a
                            ex af,af'
                                ld a,c
                                ld i,a
                            ex af,af'
                            exx
                                call PutShift
                            exx
                        endif
                        inc hl
                    pop bc
                    ld a,c
                    add a,6
                    cp 8
                    if nc
                        exx
                            inc hl
                        exx
                        sub 8
                    endif
                    ld c,a
                dwnz
                exx
                    push bc
                        ld bc,VramRowSize-VVramWidth*3/4
                        add hl,bc
                    pop bc
                exx
            pop bc
            dec c
        while nz | wend
    pop bc | pop de | pop hl | pop af
ret


; void PutShift(word vram, byte xMod, byte c);
cseg
PutShift_: public PutShift_
    push af | push hl | push de | push bc
        ld a,1 | ld (PlainBit),a
        ld a,1 | ld (PlainIndex),a
        ld a,e
        ld i,a
        ld a,c
        call PutShift
    pop bc | pop de | pop hl | pop af
ret


dseg
CurrentVram:
    defw 0
CharShift: public CharShift
    defb 0
; TextPlain_: public TextPlain_
;     defb 0

; void Locate(word vram, bool shift);
cseg
Locate_: public Locate_
    push af
        ld (CurrentVram),hl
        ld a,e
        ld (CharShift),a
    pop af
ret


; void PrintC(byte c);
cseg
PrintC_: public PrintC_
    push af | push hl | push de | push bc
        sub ' '
        ld l,a
        ld h,0
        add hl,hl ;*2
        add hl,hl ;*4
        ld de,Ascii_
        add hl,de
        ex de,hl

        ld a,07h | out (WritePlain),a
        ld a,1 | out (ReadPlain),a

        ld hl,(CurrentVram)
        ld a,(CharShift)
        or a
        if z
            ld b,4
            do
                push bc
                    ld a,(de) | inc de
                    ld c,a
                    push de
                        ld de,VramWidth

                        and 0fh
                        ld b,a
                        ld a,(hl)
                        and 0f0h
                        or b
                        ld (hl),a
                        add hl,de

                        ld a,c
                        rrca | rrca | rrca | rrca
                        and 0fh
                        ld b,a
                        ld a,(hl)
                        and 0f0h
                        or b
                        ld (hl),a
                        add hl,de
                    pop de
                pop bc
            dwnz
        else
            ld b,4
            do
                push bc
                    ld a,(de) | inc de
                    ld c,a
                    push de
                        ld de,VramWidth

                        rlca | rlca | rlca | rlca
                        and 0f0h
                        ld b,a
                        ld a,(hl)
                        and 0fh
                        or b
                        ld (hl),a
                        add hl,de

                        ld a,c
                        and 0f0h
                        ld b,a
                        ld a,(hl)
                        and 0fh
                        or b
                        ld (hl),a
                        add hl,de
                    pop de
                pop bc
            dwnz
        endif
        ld a,(CharShift)
        xor 1
        ld (CharShift),a
        if z
            ld hl,(CurrentVram)
            inc hl
            ld (CurrentVram),hl
        endif
    pop bc | pop de | pop hl | pop af
ret
